<?php

namespace App\Tools;

use App\Key\K;
use EasySwoole\Component\TableManager;
use Swlib\SaberGM;

class Proxy
{
    static $error = '';

    public static function getError()
    {
        return static::$error;
    }

    public static function parseUrl($url)
    {
        try {
            $data = SaberGM::get($url);
            $data = $data->getBody()->getContents();
            $data = base64_decode($data);
            if (!$data) {
                //如果没有获取到订阅的信息,有可能是当前的这个订阅不允许使用大陆之外的IP来请求，尝试使用下面的地址来获取一次结果
                $data=SaberGM::post('http://fast.zhangwei.fun/index/index/curlSubscribe',['url'=>$url]);
                $data = $data->getBody()->getContents();
                System::log($data,'fast.zhangwei.fun get subscribe data');
                $data = base64_decode($data);
            }
            $data = explode(PHP_EOL, $data);
            $data = array_filter($data);
            $data = array_map(function ($v) {
                $v = explode('//', $v);
                return json_decode(base64_decode($v['1']), true);
            }, $data);
            if (!$data) {
                System::log($url, '没有获取到订阅信息!');
                return false;
            }
            $parseData = [];
            foreach ($data as $v) {
                $key = $v['path'] . '_' . $v['add'] . '_' . $v['port'] . '_' . $v['host'];
                $parseData[$key] = $v;
            }
            return $parseData;
        } catch (\Throwable $exception) {
            System::log($exception->getMessage(), 'parse proxy url error');
            return false;
        }
    }

    public static function getBaseTemplate()
    {
        try {
            $data = SaberGM::get(K::MANAGE_SERVER_ADDR . '/xt/proxy-base.json', ['timeout' => 2]);
            return $data->getBody()->getContents();
        } catch (\Throwable $exception) {
            System::log($exception->getMessage(), 'get proxy template error');
            return false;
        }
    }

    public static function getAllProxy($all = false)
    {
        try {
            if ($all) {
                //返回全部的代理，包含了过期的，或者是已经超过限制了的
                $url = K::MANAGE_SERVER_ADDR_9502 . '/google/subscribe/index?bigemap=1&all=1';
            } else {
                $url = K::MANAGE_SERVER_ADDR_9502 . '/google/subscribe/index?bigemap=1';
            }
            $data = SaberGM::get($url, ['timeout' => 3]);
            $data = $data->getBody()->getContents();
            $data = json_decode(trim($data), true);
            if (!$data) {
                System::log($data, 'decode proxy error');
                return false;
            }
            return $data['data'];
        } catch (\Throwable $exception) {
            System::log($exception->getMessage(), 'get proxy template error');
            return false;
        }
    }

    //在比如测试的过程中，发现当前的服务器被封了，那么要通知主服务器
    public static function noticeForbiddenServer($key = '')
    {
        if (!$key) return;
        try {
            //由于在这是里面只能获取到代理的key值，那么将代理的key值提交给主服务器
            SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/google/api/proxy', ['code' => '403_key', 'result' => $key], ['timeout' => 3]);
        } catch (\Throwable $exception) {

        }
    }

    public static function startProxy($template, $config, $port)
    {
        if (!$config['tls']) $config['tls'] = 'none';
        $baseConfig = str_replace(
            ['{PROXY_LOCAL_PORT}', '{PROXY_PATH}', '{PROXY_HOST}', '{PROXY_ADDR}', '{PROXY_ID}', '{PROXY_AID}', '{PROXY_PORT}', '{PROXY_NET}', '{TLS}'],
            [$port, $config['path'], $config['host'], $config['add'], $config['id'], $config['aid'], $config['port'], $config['net'], $config['tls']],
            $template);
        $cachePath = System::config(K::KEY_CACHE_PATH);
        //所有的代理都是通过config+端口号来操作的，停止也是类似
        $testConfig = $cachePath . '/v2ray/config' . $port;
        if (!Shell::shellKill('v2ray/config' . $port)) {
            static::$error = '停止旧的程序失败!';
            System::log('stop old proxy failed');
            return false;
        }
        file_put_contents($testConfig, $baseConfig);
        $v2rayPath = $cachePath . '/v2ray/v2ray';
        if (!Shell::shellStart($v2rayPath . ' --config=' . $testConfig)) {
            static::$error = '启动测试v2ray失败!';
            System::log('start test proxy failed');
            return false;
        }
        return true;
    }

    public static function getProxyIp($proxy = '')
    {
        $config = System::config();
        $ip = '';
        $httpConfig = ['timeout' => 2];
        if ($proxy) {
            $httpConfig['proxy'] = $proxy;
        } elseif ($config[K::KEY_PROXY_HISTORY_ENABLE] && $config[K::KEY_PROXY_HISTORY]) {
            $httpConfig['proxy'] = $config[K::KEY_PROXY_HISTORY];
        } else {
            System::log('no proxy');
            return $ip;
        }
        try {
            $proxyIp = SaberGM::get('http://google.zhangwei.fun:9502/google/index/ip', $httpConfig);
            $ip = $proxyIp->getBody()->getContents();
            if (!filter_var($ip, FILTER_VALIDATE_IP)) $ip = '';
        } catch (\Throwable $exception) {
            System::log($exception->getMessage(), 'get proxy ip error current proxy ' . $config[K::KEY_PROXY_HISTORY]);
        }
        return $ip;
    }

    public static function checkIsPrimary($ps = '')
    {
        $primary = false;
        foreach (['香港', '狮城', '新加坡', '日本', '韩国', '台湾', '沪台', '沪韩', '沪日'] as $v) {
            if (strpos($ps, $v) !== false) return true;
        }
        return $primary;
    }

    public static function subServer($data = [])
    {
        $configTable = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        $cacheData = $configTable->get(K::KEY_SUB_SERVER_LIST);
        $cacheData['data'] = json_decode($cacheData['data'], true);
        if (!$cacheData['data']) $cacheData['data'] = [];
        if ($data) {
            $key = $data['host'] . '_' . $data['port'];
//            $has=false;
            $data['time'] = time();
            $cacheData['data'][$key] = $data;
            //这里保证在前台选择的时候，顺序不会出错，保存每个服务器都平均的被选择
            ksort($cacheData['data']);
            $cacheData['data'] = array_filter($cacheData['data'], function ($v) {
                //只会返回20秒之内有提交的服务器
                return time() - $v['time'] <= 20;
            });
            $cacheData['data'] = json_encode($cacheData['data']);
            $configTable->set(K::KEY_SUB_SERVER_LIST, $cacheData);
        } else {
            $server = array_filter($cacheData['data'], function ($v) {
                //只会返回20秒之内有提交的服务器
                return time() - $v['time'] <= 20;
            });
            return $server;
        }
    }

    //如果子服务器因为某种原因停止了，那么马上会删除这个子服务器
    public static function removeSubServer($port = '')
    {
        if (!$port) return;
        $configTable = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        $cacheData = $configTable->get(K::KEY_SUB_SERVER_LIST);
        $cacheData['data'] = json_decode($cacheData['data'], true);
        if (!$cacheData['data']) $cacheData['data'] = [];
        foreach ($cacheData['data'] as $k => $v) {
            if ($v['port'] == $port) {
                //直接通过端口号来删除
                unset($cacheData['data'][$k]);
                break;
            }
        }
        $cacheData['data'] = json_encode($cacheData['data']);
        $configTable->set(K::KEY_SUB_SERVER_LIST, $cacheData);

    }
}
